/**
 * eAdventure (formerly <e-Adventure> and <e-Game>) is a research project of the
 *    <e-UCM> research group.
 *
 *    Copyright 2005-2010 <e-UCM> research group.
 *
 *    You can access a list of all the contributors to eAdventure at:
 *          http://e-adventure.e-ucm.es/contributors
 *
 *    <e-UCM> is a research group of the Department of Software Engineering
 *          and Artificial Intelligence at the Complutense University of Madrid
 *          (School of Computer Science).
 *
 *          C Profesor Jose Garcia Santesmases sn,
 *          28040 Madrid (Madrid), Spain.
 *
 *          For more info please visit:  <http://e-adventure.e-ucm.es> or
 *          <http://www.e-ucm.es>
 *
 * ****************************************************************************
 *
 *  This file is part of eAdventure, version 2.0
 *
 *      eAdventure is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU Lesser General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      eAdventure is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU Lesser General Public License for more details.
 *
 *      You should have received a copy of the GNU Lesser General Public License
 *      along with eAdventure.  If not, see <http://www.gnu.org/licenses/>.
 */

package es.eucm.ead.model.elements.operations;

import es.eucm.ead.model.elements.BasicElement;
import es.eucm.ead.model.interfaces.Element;
import es.eucm.ead.model.interfaces.Param;

@Element
public class ElementField extends Operation {

	@Param
	private BasicElement element;

	@Param
	private String varName;

	@Param
	private Object defaultValue;

	public ElementField() {

	}

	public ElementField(BasicElement element, String varName) {
		this.element = element;
		this.varName = varName;
	}

	public ElementField(BasicElement element, String varName,
			Object defaultValue) {
		this.element = element;
		this.varName = varName;
		this.defaultValue = defaultValue;
	}

	/**
	 * Returns the element holding the field. If the element is {@code null}
	 * should be considered as a system variable. Can be another
	 * {@link ElementField} holding the element
	 *
	 * @return
	 */
	public BasicElement getElement() {
		return element;
	}

	public void setElement(BasicElement element) {
		this.element = element;
	}

	public void setVarName(String varName) {
		this.varName = varName;
	}

	/**
	 * Returns the variable definition for the field
	 *
	 * @return
	 */
	public String getVarName() {
		return varName;
	}

	public Object getDefaultValue() {
		return defaultValue;
	}

	public void setDefaultValue(Object defaultValue) {
		this.defaultValue = defaultValue;
	}

	@Override
	public String toString() {
		return (element != null ? element.getId() : "NULL") + "." + varName
				+ "(" + defaultValue + ")";
	}

}
