/**
 * eAdventure (formerly <e-Adventure> and <e-Game>) is a research project of the
 *    <e-UCM> research group.
 *
 *    Copyright 2005-2010 <e-UCM> research group.
 *
 *    You can access a list of all the contributors to eAdventure at:
 *          http://e-adventure.e-ucm.es/contributors
 *
 *    <e-UCM> is a research group of the Department of Software Engineering
 *          and Artificial Intelligence at the Complutense University of Madrid
 *          (School of Computer Science).
 *
 *          C Profesor Jose Garcia Santesmases sn,
 *          28040 Madrid (Madrid), Spain.
 *
 *          For more info please visit:  <http://e-adventure.e-ucm.es> or
 *          <http://www.e-ucm.es>
 *
 * ****************************************************************************
 *
 *  This file is part of eAdventure, version 2.0
 *
 *      eAdventure is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU Lesser General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      eAdventure is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU Lesser General Public License for more details.
 *
 *      You should have received a copy of the GNU Lesser General Public License
 *      along with eAdventure.  If not, see <http://www.gnu.org/licenses/>.
 */

package es.eucm.ead.model.interfaces.features;

import es.eucm.ead.model.elements.effects.Effect;
import es.eucm.ead.model.elements.extra.EAdList;
import es.eucm.ead.model.elements.extra.EAdMap;
import es.eucm.ead.model.params.guievents.EAdGUIEvent;

/**
 * Implemented by all classes with effects to GUI events.
 * 
 */
public interface WithBehavior {

	/**
	 * Returns the effects list associated with the given GUI event,
	 * {@code null} if there is no effects associated. This method shouldn't be
	 * used to add new effects to the actor. Returned list could be {@code null}
	 * 
	 * @param event
	 *            the GUI event
	 * @return the effects list associated with the given event
	 */
	EAdList<Effect> getEffects(EAdGUIEvent event);

	/**
	 * Add an effect to the list of effects associated with the
	 * {@link EAdGUIEvent}
	 * 
	 * @param event
	 *            the GUI event
	 * @param effect
	 *            the new effect to be added to the list
	 */
	void addBehavior(EAdGUIEvent event, Effect effect);

	/**
	 * Adds the given effects to the list of the events that will be executed
	 * when the given event is processed by this element
	 * 
	 * @param event
	 *            the GUI event
	 * @param effects
	 *            the list of effects
	 */
	void addBehavior(EAdGUIEvent event, EAdList<Effect> effects);

	/**
	 * 
	 * @return the behavior of the element
	 */
	EAdMap<EAdList<Effect>> getBehavior();

	/**
	 * Sets the behavior for the element
	 * 
	 * @param behavior
	 *            the behavior
	 */
	void setBehavior(EAdMap<EAdList<Effect>> behavior);

}
